# -*- coding: utf-8 -*-
#    original ml-10-01-iris-deep-learn.py
#    reference aidiary.hatenablog.com/entry/20161108/1478609028
#   modified by Masayuki Kogoshi & Sumio Kogoshi
#      (for learn)
#***************************************************************
import keras
from keras.utils import np_utils
from keras.models import Sequential, model_from_json
from keras.layers import Dense, Activation, Dropout
from sklearn import model_selection
from sklearn.preprocessing import StandardScaler
import numpy as np
import matplotlib.pyplot as plt
import sys
import pandas as pd
from pandas import DataFrame


# 学習結果を保存するファイルの決定
#if len(sys.argv)==1:
#    print('使用法: python iris-learn.py 保存ファイル名.h5')
#    sys.exit()
#savefile = sys.argv[1]

#savefile = iris.h5

# アヤメのデータをロードし、変数iris_dataに格納
iris_data = pd.read_csv("iris_train.csv",sep=",",header=0)

# 特徴量のセットを変数Xに、ターゲットを変数yに格納
X = DataFrame(iris_data.drop("class",axis=1))
y1 = DataFrame(iris_data["class"])
y2 = np.array(y1.T)
y = y2[0]

#standardization of x data　今回のアヤメ分類では不用だが残してあります
scaler = StandardScaler()
scaler.fit(X)
X = scaler.transform(X)

# サンプル数、特徴量の次元、クラス数の取り出し
(n_samples, n_features) = X.shape
n_classes = len(np.unique(y))

# ターゲットyをkeras用の形式に変換
y_keras = np_utils.to_categorical(y, n_classes)

# 4層ニューラルネットワークを定義
model = Sequential()
# 中間層1（units,ニューロン100個）と入力層を定義
model.add(Dense(units=100, activation='relu', input_shape=(n_features,)))
# Dropout層を定義
model.add(Dropout(0.1))
# 中間層2（units,ニューロン64個）を定義
model.add(Dense(units=64, activation='relu'))
# Dropout層を定義
model.add(Dropout(0.1))
# 出力層を定義（ニューロン数はクラス数に等しい3個）
model.add(Dense(units=n_classes, activation='softmax'))

# モデルのコンパイル
model.compile(loss='categorical_crossentropy', optimizer='adam', metrics=['accuracy'])

# モデルの学習
history = model.fit(X, y_keras, epochs=500, validation_split=0.1, batch_size=n_samples, verbose=2)

# 結果の表示
result1 = model.predict(X)
result = np.argmax(result1,axis=1)

print('ターゲット')
print(y)
print('ディープラーニングによる予測')
print(result)

# データ数をtotalに格納
total = len(X)
# ターゲット（正解）と予測が一致した数をsuccessに格納
success = sum(result==y)

# 正解率をパーセント表示
print('正解率')
print(100.0*success/total)

# 学習済AIモデルを保存
#model.save(savefile)
model_arc_json = model.to_json()
open("model_architecture.json","w").write(model_arc_json)
model.save_weights("weights.hdf5")

# 損失関数のグラフの軸ラベルを設定
plt.xlabel('time step')
plt.ylabel('loss')

# グラフ縦軸の範囲を0以上と定める
plt.ylim(0, max(np.r_[history.history['val_loss'], history.history['loss']]))

# 損失関数の時間変化を描画
val_loss, = plt.plot(history.history['val_loss'], c='#56B4E9')
loss, = plt.plot(history.history['loss'], c='#E69F00')

# グラフの凡例（はんれい）を追加
plt.legend([loss, val_loss], ['loss', 'val_loss'])

# 描画したグラフを表示
plt.show()
